package com.xy.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.dto.ExcelDTO;
import com.xy.dto.FileExportDto;
import com.xy.entity.FileExport;
import com.xy.enums.FileExportStatus;
import com.xy.enums.FileExportType;
import com.xy.mapper.FileExportMapper;
import com.xy.util.ExcelUtils;
import com.xy.util.FileUtils;
import com.xy.utils.AliOssUtil;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Cleanup;
import lombok.RequiredArgsConstructor;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 文件导出 服务实现类
 * </p>
 *
 * @author 谭斌
 * @since 2023-03-04
 */
@Service
@RequiredArgsConstructor
@Api(tags = "文件导出")
public class FileExportServiceImpl extends ServiceImpl<FileExportMapper, FileExport> implements FileExportService {

    private final FileUtils fileUtils;

   
    @Override
    @ApiOperation("分页查询")
    public R<PageBean<FileExportDto.Vo>> page(FileExportDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<FileExport> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, FileExport.class).build();
        IPage<FileExport> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(FileExportDto.Vo.class, iPage));
    }

    @Override
    @ApiOperation("添加")
    public R<FileExportDto.Vo> save(@Validated FileExportDto.Save save) {
        FileExport saveInfo = copy(FileExport.class, save);
        save(saveInfo);
        return R.ok(BeanUtil.copyProperties(saveInfo, FileExportDto.Vo.class));
    }


    @Override
    @ApiOperation("更新文件导出状态")
    public R<Boolean> updateExportStatus(Long id, String exportStatus) {
        FileExport update = new FileExport().setId(id).setExportStatus(exportStatus);
        return R.ok(this.updateById(update));
    }

    @Override
    @ApiOperation("更新文件导出状态")
    public R<Boolean> updateExportStatus(Long id, String exportStatus, String fileUrl) {
        FileExport update = new FileExport().setId(id).setExportStatus(exportStatus).setFileUrl(fileUrl);
        return R.ok(this.updateById(update));
    }


    /**
     * excel导出
     *
     * @param excelDTO
     * @param <T>
     */
    @Override
    @Async
    public <T> void exportExcelAsync(ExcelDTO<T> excelDTO) {
        String sheetName = excelDTO.getSheetName();
        Class<T> head = excelDTO.getHead();
        List<T> data = excelDTO.getData();
        FileExportType fileExportType = excelDTO.getFileExportType();
        //新增导出记录
        long id = YitIdHelper.nextId();
        String fileName = fileUtils.getFileName(id);
        String pathName = fileUtils.getPathName(fileName);
        try {
            FileExportDto.Save fileExport = new FileExportDto.Save().setId(id)
                    .setExportStatus(FileExportStatus.EXPORT_ING.getCode())
                    .setFileType(fileExportType.getCode()).setFileName(fileName);
            this.save(fileExport);
            @Cleanup OutputStream out = new FileOutputStream(pathName);
            //本地生成文件
            ExcelUtils.writeOutputStream(out, sheetName, head, data);
            this.updateExportStatus(id, FileExportStatus.EXPORT_LOCAL_SUCESS.getCode());

            FileInputStream fis = new FileInputStream(pathName);
            //对象存储云端上传
            String cloudUrl = AliOssUtil.upload(fis, ExcelUtils.EXCEL_PREFIX + fileName);
            if (StrUtil.isEmpty(cloudUrl)) {
                this.updateExportStatus(id, FileExportStatus.EXPORT_FAILED.getCode());
                return;
            }
            //更新状态与文件地址
            this.updateExportStatus(id, FileExportStatus.EXPORT_CLOUD_SUCESS.getCode(), cloudUrl);
            //删除本地文件
            fileUtils.deleteFile(pathName);

        } catch (Exception e) {
            this.updateExportStatus(id, FileExportStatus.EXPORT_FAILED.getCode());
        }
    }

    @ApiOperation("删除excel")
    @Override
    @Transactional(rollbackFor = Exception.class)
    public R<Boolean> del(FileExportDto.Del del) {
        List<Long> ids = del.getIds();
        List<FileExport> fileExports = this.listByIds(ids);
        fileExports.forEach(f -> AliOssUtil.delete(f.getFileName()));
        return R.ok(this.removeBatchByIds(ids));
    }

}
